@@ -9,6 +9,10 @@ use async_std::{
     task::{Context, Poll},
 };
 use cap_primitives::{ambient_authority, AmbientAuthority};
+#[cfg(not(windows))]
+use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
+#[cfg(windows)]
+use io_lifetimes::{AsHandle, BorrowedHandle, FromHandle, IntoHandle, OwnedHandle};
 use std::{fmt, pin::Pin};
 use unsafe_io::OwnsRaw;
 #[cfg(windows)]
@@ -111,6 +115,14 @@ impl FromRawFd for File {
     }
 }
 
+#[cfg(not(windows))]
+impl FromFd for File {
+    #[inline]
+    fn from_fd(fd: OwnedFd) -> Self {
+        Self::from_std(fs::File::from_fd(fd), ambient_authority())
+    }
+}
+
 #[cfg(windows)]
 impl FromRawHandle for File {
     #[inline]
@@ -119,6 +131,14 @@ impl FromRawHandle for File {
     }
 }
 
+#[cfg(windows)]
+impl FromHandle for File {
+    #[inline]
+    fn from_handle(handle: OwnedHandle) -> Self {
+        Self::from_std(fs::File::from_handle(handle), ambient_authority())
+    }
+}
+
 #[cfg(not(windows))]
 impl AsRawFd for File {
     #[inline]
@@ -127,6 +147,14 @@ impl AsRawFd for File {
     }
 }
 
+#[cfg(not(windows))]
+impl AsFd for File {
+    #[inline]
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        self.cap_std.as_fd()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawHandle for File {
     #[inline]
@@ -135,6 +163,14 @@ impl AsRawHandle for File {
     }
 }
 
+#[cfg(windows)]
+impl AsHandle for File {
+    #[inline]
+    fn as_handle(&self) -> BorrowedHandle<'_> {
+        self.cap_std.as_handle()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawHandleOrSocket for File {
     #[inline]
@@ -151,6 +187,14 @@ impl IntoRawFd for File {
     }
 }
 
+#[cfg(not(windows))]
+impl IntoFd for File {
+    #[inline]
+    fn into_fd(self) -> OwnedFd {
+        self.cap_std.into_fd()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawHandle for File {
     #[inline]
@@ -159,6 +203,14 @@ impl IntoRawHandle for File {
     }
 }
 
+#[cfg(windows)]
+impl IntoHandle for File {
+    #[inline]
+    fn into_handle(self) -> OwnedHandle {
+        self.cap_std.into_handle()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawHandleOrSocket for File {
     #[inline]
