@@ -3,6 +3,10 @@ use crate::net::{Incoming, SocketAddr, TcpStream};
 use async_std::os::unix::io::{AsRawFd, FromRawFd, IntoRawFd, RawFd};
 use async_std::{io, net};
 use cap_primitives::{ambient_authority, AmbientAuthority};
+#[cfg(not(windows))]
+use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
+#[cfg(windows)]
+use io_lifetimes::{AsSocket, BorrowedSocket, FromSocket, IntoSocket, OwnedSocket};
 use std::fmt;
 use unsafe_io::OwnsRaw;
 #[cfg(windows)]
@@ -84,6 +88,14 @@ impl FromRawFd for TcpListener {
     }
 }
 
+#[cfg(not(windows))]
+impl FromFd for TcpListener {
+    #[inline]
+    fn from_fd(fd: OwnedFd) -> Self {
+        Self::from_std(net::TcpListener::from_fd(fd), ambient_authority())
+    }
+}
+
 #[cfg(windows)]
 impl FromRawSocket for TcpListener {
     #[inline]
@@ -95,6 +107,14 @@ impl FromRawSocket for TcpListener {
     }
 }
 
+#[cfg(windows)]
+impl FromSocket for TcpListener {
+    #[inline]
+    fn from_socket(socket: OwnedSocket) -> Self {
+        Self::from_std(net::TcpListener::from_socket(socket), ambient_authority())
+    }
+}
+
 #[cfg(not(windows))]
 impl AsRawFd for TcpListener {
     #[inline]
@@ -103,6 +123,14 @@ impl AsRawFd for TcpListener {
     }
 }
 
+#[cfg(not(windows))]
+impl AsFd for TcpListener {
+    #[inline]
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        self.std.as_fd()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawSocket for TcpListener {
     #[inline]
@@ -111,6 +139,14 @@ impl AsRawSocket for TcpListener {
     }
 }
 
+#[cfg(windows)]
+impl AsSocket for TcpListener {
+    #[inline]
+    fn as_socket(&self) -> BorrowedSocket<'_> {
+        self.std.as_socket()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawHandleOrSocket for TcpListener {
     #[inline]
@@ -127,6 +163,14 @@ impl IntoRawFd for TcpListener {
     }
 }
 
+#[cfg(not(windows))]
+impl IntoFd for TcpListener {
+    #[inline]
+    fn into_fd(self) -> OwnedFd {
+        self.std.into_fd()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawSocket for TcpListener {
     #[inline]
@@ -135,6 +179,14 @@ impl IntoRawSocket for TcpListener {
     }
 }
 
+#[cfg(windows)]
+impl IntoSocket for TcpListener {
+    #[inline]
+    fn into_socket(self) -> OwnedSocket {
+        self.std.into_socket()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawHandleOrSocket for TcpListener {
     #[inline]
