@@ -7,6 +7,10 @@ use async_std::{
     task::{Context, Poll},
 };
 use cap_primitives::{ambient_authority, AmbientAuthority};
+#[cfg(not(windows))]
+use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
+#[cfg(windows)]
+use io_lifetimes::{AsSocket, BorrowedSocket, FromSocket, IntoSocket, OwnedSocket};
 use std::{fmt, pin::Pin};
 use unsafe_io::OwnsRaw;
 #[cfg(windows)]
@@ -129,6 +133,14 @@ impl FromRawFd for TcpStream {
     }
 }
 
+#[cfg(not(windows))]
+impl FromFd for TcpStream {
+    #[inline]
+    fn from_fd(fd: OwnedFd) -> Self {
+        Self::from_std(net::TcpStream::from_fd(fd), ambient_authority())
+    }
+}
+
 #[cfg(windows)]
 impl FromRawSocket for TcpStream {
     #[inline]
@@ -137,6 +149,14 @@ impl FromRawSocket for TcpStream {
     }
 }
 
+#[cfg(windows)]
+impl FromSocket for TcpStream {
+    #[inline]
+    fn from_socket(socket: OwnedSocket) -> Self {
+        Self::from_std(net::TcpStream::from_socket(socket), ambient_authority())
+    }
+}
+
 #[cfg(not(windows))]
 impl AsRawFd for TcpStream {
     #[inline]
@@ -145,6 +165,14 @@ impl AsRawFd for TcpStream {
     }
 }
 
+#[cfg(not(windows))]
+impl AsFd for TcpStream {
+    #[inline]
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        self.std.as_fd()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawSocket for TcpStream {
     #[inline]
@@ -153,6 +181,14 @@ impl AsRawSocket for TcpStream {
     }
 }
 
+#[cfg(windows)]
+impl AsSocket for TcpStream {
+    #[inline]
+    fn as_socket(&self) -> BorrowedSocket<'_> {
+        self.std.as_socket()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawHandleOrSocket for TcpStream {
     #[inline]
@@ -169,6 +205,14 @@ impl IntoRawFd for TcpStream {
     }
 }
 
+#[cfg(not(windows))]
+impl IntoFd for TcpStream {
+    #[inline]
+    fn into_fd(self) -> OwnedFd {
+        self.std.into_fd()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawSocket for TcpStream {
     #[inline]
@@ -177,6 +221,14 @@ impl IntoRawSocket for TcpStream {
     }
 }
 
+#[cfg(windows)]
+impl IntoSocket for TcpStream {
+    #[inline]
+    fn into_socket(self) -> OwnedSocket {
+        self.std.into_socket()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawHandleOrSocket for TcpStream {
     #[inline]
