@@ -3,6 +3,10 @@ use crate::net::{Ipv4Addr, Ipv6Addr, SocketAddr};
 use async_std::os::unix::io::{AsRawFd, FromRawFd, IntoRawFd, RawFd};
 use async_std::{io, net};
 use cap_primitives::{ambient_authority, AmbientAuthority};
+#[cfg(not(windows))]
+use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
+#[cfg(windows)]
+use io_lifetimes::{AsSocket, BorrowedSocket, FromSocket, IntoSocket, OwnedSocket};
 use std::fmt;
 use unsafe_io::OwnsRaw;
 #[cfg(windows)]
@@ -225,6 +229,14 @@ impl FromRawFd for UdpSocket {
     }
 }
 
+#[cfg(not(windows))]
+impl FromFd for UdpSocket {
+    #[inline]
+    fn from_fd(fd: OwnedFd) -> Self {
+        Self::from_std(net::UdpSocket::from_fd(fd), ambient_authority())
+    }
+}
+
 #[cfg(windows)]
 impl FromRawSocket for UdpSocket {
     #[inline]
@@ -233,6 +245,14 @@ impl FromRawSocket for UdpSocket {
     }
 }
 
+#[cfg(windows)]
+impl FromSocket for UdpSocket {
+    #[inline]
+    fn from_socket(socket: OwnedSocket) -> Self {
+        Self::from_std(net::UdpSocket::from_socket(socket), ambient_authority())
+    }
+}
+
 #[cfg(not(windows))]
 impl AsRawFd for UdpSocket {
     #[inline]
@@ -241,6 +261,14 @@ impl AsRawFd for UdpSocket {
     }
 }
 
+#[cfg(not(windows))]
+impl AsFd for UdpSocket {
+    #[inline]
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        self.std.as_fd()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawSocket for UdpSocket {
     #[inline]
@@ -249,6 +277,14 @@ impl AsRawSocket for UdpSocket {
     }
 }
 
+#[cfg(windows)]
+impl AsSocket for UdpSocket {
+    #[inline]
+    fn as_socket(&self) -> BorrowedSocket<'_> {
+        self.std.as_socket()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawHandleOrSocket for UdpSocket {
     #[inline]
@@ -265,6 +301,14 @@ impl IntoRawFd for UdpSocket {
     }
 }
 
+#[cfg(not(windows))]
+impl IntoFd for UdpSocket {
+    #[inline]
+    fn into_fd(self) -> OwnedFd {
+        self.std.into_fd()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawSocket for UdpSocket {
     #[inline]
@@ -273,6 +317,14 @@ impl IntoRawSocket for UdpSocket {
     }
 }
 
+#[cfg(windows)]
+impl IntoSocket for UdpSocket {
+    #[inline]
+    fn into_socket(self) -> OwnedSocket {
+        self.std.into_socket()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawHandleOrSocket for UdpSocket {
     #[inline]
