@@ -7,6 +7,7 @@ use async_std::{
     },
 };
 use cap_primitives::{ambient_authority, AmbientAuthority};
+use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
 use std::fmt;
 use unsafe_io::OwnsRaw;
 
@@ -156,6 +157,13 @@ impl FromRawFd for UnixDatagram {
     }
 }
 
+impl FromFd for UnixDatagram {
+    #[inline]
+    fn from_fd(fd: OwnedFd) -> Self {
+        Self::from_std(unix::net::UnixDatagram::from_fd(fd), ambient_authority())
+    }
+}
+
 impl AsRawFd for UnixDatagram {
     #[inline]
     fn as_raw_fd(&self) -> RawFd {
@@ -163,6 +171,13 @@ impl AsRawFd for UnixDatagram {
     }
 }
 
+impl AsFd for UnixDatagram {
+    #[inline]
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        self.std.as_fd()
+    }
+}
+
 impl IntoRawFd for UnixDatagram {
     #[inline]
     fn into_raw_fd(self) -> RawFd {
@@ -170,6 +185,13 @@ impl IntoRawFd for UnixDatagram {
     }
 }
 
+impl IntoFd for UnixDatagram {
+    #[inline]
+    fn into_fd(self) -> OwnedFd {
+        self.std.into_fd()
+    }
+}
+
 // Safety: `UnixDatagram` wraps a `net::UnixDatagram` which owns its handle.
 unsafe impl OwnsRaw for UnixDatagram {}
 
